/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F198B_CMP
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.1.1 Build ID : 201608241332
 * Device Ver.  : HY16F_RDSp3_DeviceV0.2 crt0.o for HY16F19xB MCU.
 * Library Ver. : 1.3
 * MCU Device   :
 * Description  :
 * Created Date : 2017/2/18
 * Created by   :
 *
 * Program Description:
 * --------------------
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "DrvCMP.h"
#include "DrvGPIO.h"
#include "DrvLCD.h"
#include "DrvREG32.h"
#include "HY16F198.h"
#include "System.h"
#include "my define.h"
#include "Display.h"
#include "DrvPMU.h"
/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_CMPdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
  };
} MCUSTATUS;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define CMPO_PORT E_PT1
#define CMPOut BIT7

//#define V12_Voltage_mueasure
//#define RLO_Voltage_mueasure
#define CH3_Compare_CH2_Voltage

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
MCUSTATUS  MCUSTATUSbits;

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);

/*----------------------------------------------------------------------------*/
/* MAIN  function                                                             */
/*----------------------------------------------------------------------------*/
int main(void)
{

  //Display Setting
  DisplayInit();
  ClearLCDframe();
  DisplayHYcon();

  //GPIO Setting & GPIO Setting
  DrvGPIO_Open(CMPO_PORT,CMPOut,E_IO_OUTPUT);  //Set PT1_7 OUTPUT

#if defined(RLO_Voltage_mueasure)
  DrvCMP_PInput(E_CH1);        //CMP positive input CH1
  DrvCMP_NInput(E_RLO);        //CMP negative input RLO
  DrvCMP_RLO_refV(2,1);        //RLO=VDD3V, CPIS=Closed
  DrvCMP_InputSwitch(1);       //INPUT SHORT SWITCH ENABLE
#endif

#if defined(V12_Voltage_mueasure)
  DrvPMU_VDDA_Voltage(E_VDDA2_4);
  DrvPMU_VDDA_LDO_Ctrl(E_LDO);
  DrvPMU_BandgapEnable();
  DrvPMU_REFO_Enable();
  DrvCMP_PInput(E_V12);        //CMP positive input V12
  DrvCMP_NInput(E_CH1);        //CMP negative input CH1
  DrvCMP_InputSwitch(1);       //INPUT SHORT SWITCH ENABLE
#endif

#if defined(CH3_Compare_CH2_Voltage)
  unsigned int i=0;
  DrvCMP_PInput(E_CH3);        //CMP positive input CH3
  DrvCMP_NInput(E_CH2);        //CMP negative input CH2
  DrvCMP_InputSwitch(0);       //INPUT SHORT SWITCH ENABLE
#endif

  DrvCMP_Enable();             //CMP enable.
  DrvCMP_OutputPinEnable(0);   //Enable CMP digital output to port
                               //0 : PT1.7

#if defined(RLO_Voltage_mueasure)
  // Setting CPDM=0000b and CPDA to observe RLO voltage
  DrvCMP_RLO_Ctrl(0x00,0x00);  //CH1 to VSS around 0.0062V
  DrvCMP_RLO_Ctrl(0x01,0x00);  //CH1 to VSS around 0.2132V
  DrvCMP_RLO_Ctrl(0x02,0x00);  //CH1 to VSS around 0.4163V
  DrvCMP_RLO_Ctrl(0x03,0x00);  //CH1 to VSS around 0.6205V
  DrvCMP_RLO_Ctrl(0x04,0x00);  //CH1 to VSS around 0.8221V
  DrvCMP_RLO_Ctrl(0x05,0x00);  //CH1 to VSS around 1.0252V
  DrvCMP_RLO_Ctrl(0x06,0x00);  //CH1 to VSS around 1.2290V
  DrvCMP_RLO_Ctrl(0x07,0x00);  //CH1 to VSS around 1.4355V
  DrvCMP_RLO_Ctrl(0x08,0x00);  //CH1 to VSS around 1.6393V
  DrvCMP_RLO_Ctrl(0x09,0x00);  //CH1 to VSS around 1.8449V
  DrvCMP_RLO_Ctrl(0x0A,0x00);  //CH1 to VSS around 2.0476V
  DrvCMP_RLO_Ctrl(0x0B,0x00);  //CH1 to VSS around 2.2535V
  DrvCMP_RLO_Ctrl(0x0C,0x00);  //CH1 to VSS around 2.4577V
  DrvCMP_RLO_Ctrl(0x0D,0x00);  //CH1 to VSS around 2.6613V
  DrvCMP_RLO_Ctrl(0x0E,0x00);  //CH1 to VSS around 2.8686V
  DrvCMP_RLO_Ctrl(0x0F,0x00);  //CH1 to VSS around 3.0777V, VDD3V around 3.3091V
#endif

#if defined(CH3_Compare_CH2_Voltage)
  while(1)
  {
    i=DrvCMP_ReadData();
    LCD_DATA_DISPLAY(i);       //IF positive(CH3) > negative(CH2), CMPO=High, LCD Display(1). otherwise CMPO=Low, LCD Display(0).
  }
#endif

  while(1);
  return 0;
}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : CMP & OPA interrupt Service Routine (HW3).                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{
  MCUSTATUSbits.b_CMPdone=1;
  DrvCMP_ClearIntFlag();       //Clear CMP interrupt flag
}
/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: tlb_exception_handler()                                     */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void tlb_exception_handler()
{
  asm("nop");                  //procedure define by customer.
  asm("nop");
}
/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
