/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F198B_I2C
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.1.1 Build ID : 201608241332
 * Device Ver.  : HY16F_RDSp3_DeviceV0.2 crt0.o for HY16F19xB MCU.
 * Library Ver. : 1.3
 * MCU Device   :
 * Description  :
 * Created Date : 2018/2/18
 * Created by   :
 *
 * Program Description:
 * --------------------
 *
 * HY16H198
 * ---------
 *          |              --------
 *    PT2.3 | SDA <--> SDA |EEPROM|
 *    PT2.2 | SCK ---> SCK --------
 *          |
 *    PT2.4 | LXIN
 *    PT2.5 | LXOUT
 *     GND  |
 *          |
 * ---------
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY16F198.h"
#include "DrvI2C.h"
#include "System.h"
#include "EEPROM_24Cxx.h"
#include "my define.h"

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define  I2C_PORT E_PT2
#define  SCL_PIN 2
#define  SDA_PIN 3
#define  SCL_BIT BIT2
#define  SDA_BIT BIT3
#define  I2CBufferSize  256
#define  I2C_WRITE  0x00       // I2C WRITE command
#define  I2C_READ   0x01       // I2C READ command
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char I2C_Read_Buffer[I2CBufferSize];;
unsigned char I2C_RW;
unsigned char I2C_TARGET;      // Target I2C slave address
unsigned char I2C_Sendbuf[I2CBufferSize];
unsigned char I2C_Recbuf[I2CBufferSize];
unsigned char I2C_EndFlag;
unsigned int I2C_DataTxLen,I2C_DataTxIndex,I2C_DataRxLen,I2C_DataRxIndex;


/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{
  unsigned int i;
  unsigned char EEPROM_WriteData[4] = {0x02,0x03,0x04,0x05};

  for(i=0;i<=I2CBufferSize;i++)
   {
     I2C_Read_Buffer[i]=0;                   //Initial I2C data buffer=0
   }

  DrvI2C_SetIOPin(5);                        //Setting io pin, 5: SCL=PT2.2;SDA=PT2.3
  DrvI2C_Open(0x63);                         //Enable I2C function and set I2C baud rate=5kHz
  //Default CPU clock is 2MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 2000000/[4*(99+1)]=5kHz
  DrvI2C_EnableInt(2);                       // Enable I2C interrupt and error interrupt 

  SYS_EnableGIE(4,0x1FF);                    //Enable GIE(Global Interrupt)

  // I2C single I2C_WRITE & I2C_READ
  EEPROM_ByteWrite(0x00,0x01);               //Single Byte I2C_WRITE word address 0x00, and I2C_WRITE data 0x01
  Delay(1000);	                             //Delay for EEPROM 24C02 I2C_WRITE Cycle Time 5ms
  I2C_Read_Buffer[0]=EEPROM_ByteRead(0x00);  //Single Byte I2C_READ word address 0x00, the I2C_READ value is 0x01
  Delay(1000);                               //Delay for EEPROM 24C02 I2C_WRITE Cycle Time 5ms

  // I2C sequential I2C_WRITE & I2C_READ
  EEPROM_WriteArray(0x01,EEPROM_WriteData,4); //I2C_WRITE array data to the word address from 0x01 to 0x04
  Delay(1000);                                //Delay for EEPROM 24C02 I2C_WRITE Cycle Time 5ms
  EEPROM_ReadArray(I2C_Read_Buffer, 0x00, 5); //sequential I2C_READ data from 0x00 to 0x04
  Delay(1000);                                //Delay for EEPROM 24C02 I2C_WRITE Cycle Time 5ms

  while(1);

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{
  unsigned char I2C_Status,I2C_IntFlag;

  I2C_IntFlag=DrvI2C_ReadIntFlag();
  if((I2C_IntFlag == E_DRVI2C_INT)||(I2C_IntFlag == E_DRVI2C_INT_ALL))  //Get I2C Interrupt Flag
  {
    I2C_Status=DrvI2C_GetStatusFlag();     //Get I2C Status Flag
    switch(I2C_Status)
    {
      case 0x90:  //MACTFlag+RWFlag
          {       //START has been transmitted
            DrvI2C_WriteData(I2C_TARGET);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);          //Clear all I2C flag
            break;
          };
      case 0x84:  //MACTFlag+ACKFlag
          {       //Slave A + W has been transmitted. ACK has been received.
            DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            break;
          };
      case 0x80: //MACTFlag
          {      //Slave A + W has been transmitted. ACK has been received.
            DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            break;
          };
      case 0x30:
          {
            DrvI2C_Ctrl(0,0,0,0);      //Clear all I2C flag
            I2C_EndFlag=1;
            break;
          };
      case 0x8C:  //MACTFlag+DFFlag+ACKFlag
          {       //DATA has been transmitted and ACK has been received
            if(I2C_DataTxIndex<I2C_DataTxLen)
            {
              DrvI2C_WriteData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
              DrvI2C_Ctrl(0,0,0,0);    // Clear all I2C flag
            }
            else
            {
              if(I2C_RW == I2C_WRITE)
              {
                DrvI2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
                I2C_EndFlag=1;
              }
              else if(I2C_RW == I2C_READ)
                DrvI2C_Ctrl(1,0,0,0);  //I2C as master sends START signal
              I2C_DataTxIndex=0;
            }
            break;
          };
      case 0x88:  //MACTFlag+DFFlag
          {       //DATA has been transmitted and NACK has been received
            DrvI2C_Ctrl(0,1,0,0);    //I2C as master sends STOP signal
            I2C_DataTxIndex=0;
            I2C_EndFlag=1;
            break;
          };
      case 0xB0:
          {       //A repeated START has been transmitted.
            DrvI2C_WriteData(I2C_TARGET | I2C_READ);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);    //Clear all I2C flag
            break;
          }
      case 0x94:  //MACTFlag+RWFlag+ACKFlag
          {       //Slave A + R has been transmitted. ACK has been received.
            if(I2C_DataRxLen>1)
            {
              DrvI2C_Ctrl(0,0,0,1);  //Set ACK bit
            }else{
              DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            }
            break;
          };
      case 0x9C:  //MACTFlag+RWFlag+DFFlag+ACKFlag
          {       //Data byte has been received. ACK has been transmitted.
            I2C_Recbuf[I2C_DataRxIndex++]=DrvI2C_ReadData();
            if((I2C_DataRxLen-1)>I2C_DataRxIndex)
            {
              DrvI2C_Ctrl(0,0,0,1);  //Set ACK bit
            }
            else
            {
              DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            }
            break;
          };
      case 0x98:  //MACTFlag+RWFlag+DFFlag
          {       //Data byte has been received. NACK has been transmitted.
            I2C_Recbuf[I2C_DataRxIndex++]=DrvI2C_ReadData();
            DrvI2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
            I2C_EndFlag=1;
            break;
          };
      default:
          {
            DrvI2C_Ctrl(0,0,0,0);  //Clear all I2C flag
            I2C_EndFlag=1;
            break;
          };
    }
    DrvI2C_ClearIRQ();
    DrvI2C_ClearEIRQ();            //Clear EIRQFlag
    DrvI2C_ClearIntFlag(0);        //Clear I2C Interrupt Flag(I2CIF)
  }
  if((I2C_IntFlag == E_DRVI2C_ERROR_INT)||(I2C_IntFlag == E_DRVI2C_INT_ALL))  //Get I2C Error Interrupt Flag
  {
    I2C_EndFlag=1;
    DrvI2C_ClearIRQ();
    DrvI2C_ClearEIRQ();            //Clear EIRQFlag
    DrvI2C_ClearIntFlag(1);        //Clear I2C Interrupt Flag(I2CEIF)
    DrvI2C_Ctrl(0,0,0,0);          //Clear all I2C flag
  }
  SYS_EnableGIE(4,0x1FF);          //Enable GIE(Global Interrupt)
}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : CMP & OPA interrupt Service Routine (HW3).                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: tlb_exception_handler()                                     */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void tlb_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
}


/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
