/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F198B_Timer
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.1.1 Build ID : 201608241332
 * Device Ver.  : HY16F_RDSp3_DeviceV0.2 crt0.o for HY16F19xB MCU.
 * Library Ver. : 1.3
 * MCU Device   :
 * Description  :
 * Created Date : 2018/2/18
 * Created by   :
 *
 * Program Description:
 * --------------------
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY16F198.h"
#include "System.h"
#include "DrvClock.h"
#include "DrvGPIO.h"
#include "DrvLCD.h"
#include "DrvREG32.h"
#include "DrvTimer.h"
#include "Display.h"
#include "my define.h"

/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
  };
} MCUSTATUS;

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
MCUSTATUS  MCUSTATUSbits;
unsigned int TimerA_count, TimerB_count, TimerC0_count, TimerC1_count;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define  TMATEST
//#define  TMBTEST
//#define  TMCTEST

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);

/*----------------------------------------------------------------------------*/
/* MAIN  function                                                             */
/*----------------------------------------------------------------------------*/
int main(void)
{

  TimerA_count=0;
  TimerB_count=0;
  TimerC0_count=0;
  TimerC1_count=0;
  DisplayInit();
  ClearLCDframe();


#if defined(TMATEST)
  DrvTMA_Open(15,0);                             //Timer A Overflow
                                                 //15:taclk/65536/32;TMRDV=32
                                                 //00:HS_CK
  DrvTIMER_ClearIntFlag(E_TMA);                  //Clear Timer A interrupt flag
  DrvTIMER_EnableInt(E_TMA);                     //Timer A interrupt enable
#endif

#if defined(TMBTEST)
  DrvTMBC_Clk_Source(0,3);                       //Timer B Prescaler 1
                                                 //0: HS_CK,clock source.
                                                 //3: clock divider.8

  DrvTMB_Open(E_TMB_MODE0,E_TMB_NORMAL,0xFFFF);  //Timer B overflow 0xffff
  DrvTIMER_ClearIntFlag(E_TMB);                  //Clear Timer B interrupt flag
  DrvTIMER_EnableInt(E_TMB);                     //Timer B interrupt enable

#elif defined(TMCTEST)
  DrvTMBC_Clk_Source(0,1);                       //Timer B Prescaler 1
                                                 //0: HS_CK,clock source.
                                                 //1: clock divider.2

  DrvTMB_Open(E_TMB_MODE0,E_TMB_NORMAL,0xFFFF);  //Timer B overflow 0xffff

  DrvCapture1_Open(2,14,1);                      //Timer C0 use as Capture 1
                                                 //input source selection
                                                 //2:LS_CK
                                                 //14:16384 1:Rising-edge trigger
  DrvCapture2_Open(1,1);                         //Timer C1 use as Capture 2
                                                 //Input source selection
                                                 //1:same as Caputre1
                                                 //Falling-edge trigger

  DrvTIMER_ClearIntFlag(E_TMC0);                 //Clear TMC0 interrupt flag
  DrvTIMER_ClearIntFlag(E_TMC1);                 //Clear TMC1 interrupt flag
  DrvTIMER_EnableInt(E_TMC0);                    //Timer C0 interrupt enable
  DrvTIMER_EnableInt(E_TMC1);                    //Timer C1 interrupt enable
#endif

  SYS_EnableGIE(4,0x1FF);                        //Enable GIE(Global Interrupt)
  MCUSTATUSbits._byte = 0;

  while(1)                                       //Wait for Interrupt
  {

  if(MCUSTATUSbits.b_TMAdone==1)
  {
    LCD_DATA_DISPLAY(TimerA_count);
    MCUSTATUSbits.b_TMAdone=0;
  }

  if(MCUSTATUSbits.b_TMBdone==1)
  {
    LCD_DATA_DISPLAY(TimerB_count);
    MCUSTATUSbits.b_TMBdone=0;
  }

  if(MCUSTATUSbits.b_TMC0done==1)
  {
    LCD_DATA_DISPLAY(TimerC0_count);
    MCUSTATUSbits.b_TMC0done=0;
  }

  if(MCUSTATUSbits.b_TMC1done==1)
  {
    LCD_DATA_DISPLAY(TimerC1_count);
    MCUSTATUSbits.b_TMC1done=0;
  }

  }

  return 0;
}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{
  if(DrvTIMER_GetIntFlag(E_TMA))
  {
    DrvTIMER_ClearIntFlag(E_TMA);           // Clear TMA interrupt flag
    MCUSTATUSbits.b_TMAdone=1;
    TimerA_count++;
  }

  if(DrvTIMER_GetIntFlag(E_TMB))
  {
    DrvTIMER_ClearIntFlag(E_TMB);           // Clear TMB interrupt flag
    MCUSTATUSbits.b_TMBdone=1;
    TimerB_count++;
  }

  if(DrvTIMER_GetIntFlag(E_TMC0))
  {
    DrvTIMER_ClearIntFlag(E_TMC0);          // Clear TMC0 interrupt flag
    MCUSTATUSbits.b_TMC0done=1;
    TimerC0_count++;
  }

  if(DrvTIMER_GetIntFlag(E_TMC1))
  {
    DrvTIMER_ClearIntFlag(E_TMC1);          // Clear TMC1 interrupt flag
    MCUSTATUSbits.b_TMC1done=1;
    TimerC1_count++;
  }
}
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : CMP & OPA interrupt Service Routine (HW3).                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: tlb_exception_handler()                                     */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void tlb_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
}
/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
