/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F198B_UART
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.1.1 Build ID : 201608241332
 * Device Ver.  : HY16F_RDSp3_DeviceV0.2 crt0.o for HY16F19xB MCU.
 * Library Ver. : 1.3
 * MCU Device   :
 * Description  :
 * Created Date : 2018/2/18
 * Created by   :
 *
 * Program Description:
 * --------------------
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "DrvClock.h"
#include "DrvGPIO.h"
#include "DrvUART.h"
#include "HY16F198.h"
#include "System.h"
#include "my define.h"


/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
  };
} MCUSTATUS;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
//#define PORT9
#define HSRC    //Internal HAO
//#define HSXT  //External 4MHz

#define HAO_2MHZ
//#define HAO_4MHZ
//#define HAO_10MHZ
//#define HAO_16MHZ

#if defined(PORT9)
#define  UART_PORT E_PT9
#define  UART_TXD  BIT4
#define  UART_RXD  BIT5
#else
#define  UART_PORT E_PT2
#define  UART_TXD  BIT0
#define  UART_RXD  BIT1
#endif

#define  Uart_RX_BufferSize 6
#define  Uart_TX_BufferSize 8

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char UartRxBuffer[Uart_RX_BufferSize]={0},UartTxBuffer[Uart_TX_BufferSize]={0};
unsigned char UartTxIndex,UartTxLength,UartRxIndex,UartRxLength;
MCUSTATUS  MCUSTATUSbits;

unsigned char UartRxBuffer_Command[Uart_RX_BufferSize]=
{
0x61,0x62,0x63,0x64,0x65,0x66 //ASCII  KEYWORD = abcdef
};
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void InitalUart(void);
void InitalClock(void);
/*----------------------------------------------------------------------------*/
/* MAIN  function                                                             */
/*----------------------------------------------------------------------------*/
int main(void)
{
  unsigned char Stop_To_Send_UART=DISABLE;
  InitalClock();
  InitalUart();
  MCUSTATUSbits._byte = 0;
  MCUSTATUSbits.b_UART_TxDone=ENABLE;
  SYS_EnableGIE(4,0x1FF);  //Enable GIE(Global Interrupt)

  UartTxIndex=0;
  UartRxIndex=0;

  while(1)
  {

    //UART RX
    if(MCUSTATUSbits.b_UART_RxDone==ENABLE)
    {

      if(
        (UartRxBuffer[5]==UartRxBuffer_Command[5] && UartRxBuffer[4]==UartRxBuffer_Command[4]) &&
        (UartRxBuffer[3]==UartRxBuffer_Command[3] && UartRxBuffer[2]==UartRxBuffer_Command[2]) &&
        (UartRxBuffer[1]==UartRxBuffer_Command[1] && UartRxBuffer[0]==UartRxBuffer_Command[0])
        )
    {
        //if UART receive == 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f)
        //send out 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f) and stop to send out
        Stop_To_Send_UART=ENABLE;  //UART stop to send out ABCDEF
        for(UartTxLength=0;UartTxLength<=Uart_TX_BufferSize;UartTxLength++)
        {
          UartTxBuffer[UartTxLength]=UartRxBuffer[UartTxLength];
          if(UartTxLength==Uart_RX_BufferSize)
          {
            UartRxIndex=0;
            MCUSTATUSbits.b_UART_TxDone=DISABLE;
            DrvUART_EnableInt(ENABLE,DISABLE);   //Enable UART Tx Interrupt, Disable UART Rx Interrup
            while(!MCUSTATUSbits.b_UART_TxDone); //If  MCUSTATUSbits.b_UART_TxDone=DISABLE, stop at here
          }
        }
    }
    else
    {
      //if UART receive != 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f)
      //User defined at here
      Stop_To_Send_UART=DISABLE;   //UART start to send out ABCDEF
    }
      UartRxIndex=0;               //When finished the data reception. Set UartRxIndex=0
      MCUSTATUSbits.b_UART_RxDone=DISABLE;
    }

    //UART TX
    if(MCUSTATUSbits.b_UART_TxDone==ENABLE && Stop_To_Send_UART==DISABLE )
    {
      UartTxBuffer[7]='\r';
      UartTxBuffer[6]='\n';
      UartTxBuffer[5]=0x46;  //F
      UartTxBuffer[4]=0x45;  //E
      UartTxBuffer[3]=0x44;  //D
      UartTxBuffer[2]=0x43;  //C
      UartTxBuffer[1]=0x42;  //B
      UartTxBuffer[0]=0x41;  //A
      MCUSTATUSbits.b_UART_TxDone=DISABLE;
      UartTxLength=8;
      UartTxIndex=0;
      DrvUART_EnableInt(ENABLE,ENABLE);     //Enable UART Tx Interrupt;Enable UART Rx Interrupt
      while(!MCUSTATUSbits.b_UART_TxDone);  //If  MCUSTATUSbits.b_UART_TxDone=DISABLE, stop at here
    }

  }

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{
  if(DrvUART_GetRxFlag())
  {
    UartRxBuffer[UartRxIndex]=DrvUART_Read();
    UartRxIndex++;
    if(UartRxIndex>=Uart_RX_BufferSize)
    {
      UartRxIndex=0;
      MCUSTATUSbits.b_UART_RxDone=ENABLE;
    }
      DrvUART_ClrRxFlag();
  }

  if(DrvUART_GetTxFlag())
  {
    if(MCUSTATUSbits.b_UART_TxDone==DISABLE)
    {
      DrvUART_Write(UartTxBuffer[UartTxIndex++]);
      DrvUART_ClrTxFlag();
      if(UartTxIndex>=UartTxLength)
      {
        DrvUART_EnableInt(ENABLE,ENABLE);  //ENABLE UART Tx Interrupt, ENABLE UART Rx Interrupt
        MCUSTATUSbits.b_UART_TxDone=ENABLE;
        UartTxIndex=0;
      }
    }
    if(MCUSTATUSbits.b_UART_TxDone==ENABLE)
    {
      DrvUART_EnableInt(DISABLE,ENABLE);  //DISABLE UART Tx Interrupt, ENABLE UART Rx Interrupt
      DrvUART_ClrTxFlag();
    }
  }

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : CMP & OPA interrupt Service Routine (HW3).                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: tlb_exception_handler()                                     */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void tlb_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalClock()                                               */
/* Description  : CLOCK Initial Subroutines.                                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalClock(void)
{

#if defined(HSRC)
  #if defined(HAO_2MHZ)
  //Clock INIT 2MHZ
  DrvCLOCK_EnableHighOSC(E_INTERNAL,10);       //Select HSRC
  DrvCLOCK_SelectIHOSC(0);                     //Select internal 2MHZ
  DrvCLOCK_SelectMCUClock(0,0);                //CPU CLOCK IS 'hs_ck/1'
  DrvCLOCK_CalibrateHAO(0);                    //Calibration 1.843MHz
  #endif
  #if defined(HAO_4MHZ)
  //Clock INIT 4MHZ
  DrvCLOCK_EnableHighOSC(E_INTERNAL,10);       //Select HSRC
  DrvCLOCK_SelectIHOSC(1);                     //Select internal 4MHZ
  DrvCLOCK_SelectMCUClock(0,0);                //CPU CLOCK IS 'hs_ck/1'
  DrvCLOCK_CalibrateHAO(1);                    //Calibration 4.147MHz
  #endif
  #if defined(HAO_10MHZ)
  //Clock INIT 10MHZ
  DrvCLOCK_EnableHighOSC(E_INTERNAL,10);       //Select HSRC
  DrvCLOCK_SelectIHOSC(2);                     //Select internal 10MHZ
  DrvCLOCK_SelectMCUClock(0,0);                //CPU CLOCK IS 'hs_ck/1'
  DrvCLOCK_CalibrateHAO(2);                    //Calibration 9.216MHz
  #endif
  #if defined(HAO_16MHZ)
  //Clock INIT 16MHZ
  DrvCLOCK_EnableHighOSC(E_INTERNAL,10);       //Select HSRC
  DrvCLOCK_SelectIHOSC(3);                     //Select internal 16MHZ
  DrvCLOCK_SelectMCUClock(0,0);                //CPU CLOCK IS 'hs_ck/1'
  DrvCLOCK_CalibrateHAO(3);                    //Calibration 15.667MHz
  #endif
#endif

#if defined(HSXT)
  DrvCLOCK_EnableHighOSC (E_EXTERNAL,50);
#endif
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalUart()                                                */
/* Description  : UART Initial Subroutines.                                   */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalUart(void)
{

#if defined(HSRC)
  DrvUART_ClkEnable(1,0);            //Choose the internal HAO as clock source
  #if defined(PORT9)
    #if defined(HAO_2MHZ)
    DrvUART_Open(1843,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,7);
    #endif
    #if defined(HAO_4MHZ)
    DrvUART_Open(4147,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,7);
    #endif
    #if defined(HAO_10MHZ)
    DrvUART_Open(9216,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,7);
    #endif
    #if defined(HAO_16MHZ)
    DrvUART_Open(15667,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,7);
  #endif
  #else //PORT2
    #if defined(HAO_2MHZ)
    DrvUART_Open(1843,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
    #endif
    #if defined(HAO_4MHZ)
    DrvUART_Open(4147,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
    #endif
    #if defined(HAO_10MHZ)
    DrvUART_Open(9216,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
    #endif
    #if defined(HAO_16MHZ)
    DrvUART_Open(15667,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
    #endif
  #endif
#endif

  //1843 : oscillator frequency 2MHz Unit After Calibration HAO = 1843kHz
  //4147 : oscillator frequency 4MHz Unit After Calibration HAO = 4147kHz
  //9216 : oscillator frequency 10MHz Unit After Calibration HAO = 9216kHz
  //15667 : oscillator frequency 16MHz Unit After Calibration HAO = 15667kHz
  //None parity
  //8 data bits.
  //7 : Port 9.4 =TX, Port 9.5 =RX
  //2 : Port 2.0 =TX, Port 2.1 =RX

#if defined(HSXT)
  DrvUART_ClkEnable(0,0);            //Choose the external 4MHz OSC as clock source
  DrvUART_Open(4000,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
#endif

#if defined(PORT9)
  DrvGPIO_LCDIOOpen(UART_PORT, UART_TXD, E_IO_OUTPUT);
  DrvGPIO_LCDIOOpen(UART_PORT, UART_RXD, E_IO_INPUT);
#else
  DrvGPIO_Open(UART_PORT,UART_TXD,E_IO_OUTPUT);
  DrvGPIO_Open(UART_PORT,UART_RXD,E_IO_INPUT);
  DrvGPIO_Open(UART_PORT,UART_RXD|UART_TXD,E_IO_PullHigh);
#endif

  DrvGPIO_ClkGenerator(E_HS_CK,1);
  DrvUART_EnableInt(ENABLE,ENABLE); //Enable UART Tx Interrupt, Enable UART Rx Interrupt
  DrvUART_Disable_AutoBaudrate();
  DrvUART_Close();
  DrvUART_Enable();
}

/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
