/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F198B_UART2
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.1.1 Build ID : 201608241332
 * Device Ver.  : HY16F_RDSp3_DeviceV0.2 crt0.o for HY16F19xB MCU.
 * Library Ver. : 1.3
 * MCU Device   :
 * Description  :
 * Created Date : 2018/2/18
 * Created by   :
 *
 * Program Description:
 * --------------------
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "DrvClock.h"
#include "DrvGPIO.h"
#include "DrvUART.h"
#include "HY16F198.h"
#include "System.h"
#include "my define.h"

/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_UART2_TxDone:1;
    unsigned b_UART2_RxDone:1;
  };
} MCUSTATUS;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
//#define PORT9
#define HSRC  //Internal HAO
//#define HSXT  //External 4MHz

#define HAO_2MHZ
//#define HAO_4MHZ
//#define HAO_10MHZ
//#define HAO_16MHZ


#if defined(PORT9)
#define  UART2_PORT E_PT9
#define  UART2_TXD  BIT6
#define  UART2_RXD  BIT7
#else
#define  UART2_PORT E_PT2
#define  UART2_TXD  BIT2
#define  UART2_RXD  BIT3
#endif

#define  Uart2_RX_BufferSize 6
#define  Uart2_TX_BufferSize 8

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char Uart2RxBuffer[Uart2_RX_BufferSize]={0},Uart2TxBuffer[Uart2_TX_BufferSize]={0};
unsigned char Uart2TxIndex,Uart2TxLength,Uart2RxIndex,Uart2RxLength;
MCUSTATUS  MCUSTATUSbits;

unsigned char Uart2RxBuffer_Command[Uart2_RX_BufferSize]=
{
0x61,0x62,0x63,0x64,0x65,0x66 //ASCII  KEYWORD = abcdef
};
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void InitalUart2(void);
void InitalClock(void);

/*----------------------------------------------------------------------------*/
/* MAIN  function                                                             */
/*----------------------------------------------------------------------------*/
int main(void)
{

  unsigned char Stop_To_Send_UART2=DISABLE;
  InitalClock();
  InitalUart2();
  MCUSTATUSbits._byte = 0;
  MCUSTATUSbits.b_UART2_TxDone=ENABLE;
  SYS_EnableGIE(4,0x1FF);  //Enable GIE(Global Interrupt)

  Uart2TxIndex=0;
  Uart2RxIndex=0;

  while(1)
  {

    //UART2 RX
    if(MCUSTATUSbits.b_UART2_RxDone==ENABLE)
    {

      if(
        (Uart2RxBuffer[5]==Uart2RxBuffer_Command[5] && Uart2RxBuffer[4]==Uart2RxBuffer_Command[4]) &&
        (Uart2RxBuffer[3]==Uart2RxBuffer_Command[3] && Uart2RxBuffer[2]==Uart2RxBuffer_Command[2]) &&
        (Uart2RxBuffer[1]==Uart2RxBuffer_Command[1] && Uart2RxBuffer[0]==Uart2RxBuffer_Command[0])
        )
    {
        //if UART2 receive == 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f)
        //send out 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f) and stop to send out
        Stop_To_Send_UART2=ENABLE;  //UART stop to send out ABCDEF
        for(Uart2TxLength=0;Uart2TxLength<=Uart2_TX_BufferSize;Uart2TxLength++)
        {
          Uart2TxBuffer[Uart2TxLength]=Uart2RxBuffer[Uart2TxLength];
          if(Uart2TxLength==Uart2_RX_BufferSize)
          {
            Uart2RxIndex=0;
            MCUSTATUSbits.b_UART2_TxDone=DISABLE;
            DrvUART2_EnableInt(ENABLE,DISABLE);    //Enable UART2 Tx Interrupt, Disable UART2 Rx Interrup
            while(!MCUSTATUSbits.b_UART2_TxDone);  //If  MCUSTATUSbits.b_UART2_TxDone=DISABLE, stop at here
          }
        }
    }
    else
    {
      //if UART2 receive != 0x61(a)0x62(b)0x63(c)0x64(d)0x65(e)0x66(f)
      //User defined at here
      Stop_To_Send_UART2=DISABLE;   //UART2 start to send out ABCDEF
    }
      Uart2RxIndex=0; //When finished the data reception. Set Uart2RxIndex=0
      MCUSTATUSbits.b_UART2_RxDone=DISABLE;
    }

    //UART2 TX
    if(MCUSTATUSbits.b_UART2_TxDone==ENABLE && Stop_To_Send_UART2==DISABLE )
    {
      Uart2TxBuffer[7]='\r';
      Uart2TxBuffer[6]='\n';
      Uart2TxBuffer[5]=0x46;  //F
      Uart2TxBuffer[4]=0x45;  //E
      Uart2TxBuffer[3]=0x44;  //D
      Uart2TxBuffer[2]=0x43;  //C
      Uart2TxBuffer[1]=0x42;  //B
      Uart2TxBuffer[0]=0x41;  //A
      MCUSTATUSbits.b_UART2_TxDone=DISABLE;
      Uart2TxLength=8;
      Uart2TxIndex=0;
      DrvUART2_EnableInt(ENABLE,ENABLE);     //Enable UART2 Tx Interrupt;Enable UART2 Rx Interrupt
      while(!MCUSTATUSbits.b_UART2_TxDone);  //If  MCUSTATUSbits.b_UART2_TxDone=DISABLE, stop at here
    }

  }

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : CMP & OPA interrupt Service Routine (HW3).                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 Interrupt Service Routines(HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

  if(DrvUART2_GetRxFlag())
  {
    Uart2RxBuffer[Uart2RxIndex]=DrvUART2_Read();
    Uart2RxIndex++;
    if(Uart2RxIndex>=Uart2_RX_BufferSize)
    {
      Uart2RxIndex=0;
      MCUSTATUSbits.b_UART2_RxDone=ENABLE;
    }
      DrvUART2_ClrRxFlag();
  }

  if(DrvUART2_GetTxFlag())
  {
    if(MCUSTATUSbits.b_UART2_TxDone==DISABLE)
    {
      DrvUART2_Write(Uart2TxBuffer[Uart2TxIndex++]);
      DrvUART2_ClrTxFlag();
      if(Uart2TxIndex>=Uart2TxLength)
      {
        DrvUART2_EnableInt(ENABLE,ENABLE);  //ENABLE UART2 Tx Interrupt, ENABLE UART2 Rx Interrupt
        MCUSTATUSbits.b_UART2_TxDone=ENABLE;
        Uart2TxIndex=0;
      }
    }
      if(MCUSTATUSbits.b_UART2_TxDone==ENABLE)
      {
        DrvUART2_EnableInt(DISABLE,ENABLE);  //DISABLE UART2 Tx Interrupt, ENABLE UART2 Rx Interrupt
        DrvUART2_ClrTxFlag();
      }

  }


}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: tlb_exception_handler()                                     */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void tlb_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
}
/*----------------------------------------------------------------------------*/
/* Function Name: InitalClock()                                               */
/* Description  : CLOCK Initial Subroutines.                                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalClock(void)
{

#if defined(HSRC)
  #if defined(HAO_2MHZ)
  //Clock INIT 2MHZ
  DrvCLOCK_EnableHighOSC(E_INTERNAL,10);       //Select HSRC
  DrvCLOCK_SelectIHOSC(0);                     //Select internal 2MHZ
  DrvCLOCK_SelectMCUClock(0,0);                //CPU CLOCK IS 'hs_ck/1'
  DrvCLOCK_CalibrateHAO(0);                    //Calibration 1.843MHz
  #endif
  #if defined(HAO_4MHZ)
  //Clock INIT 4MHZ
  DrvCLOCK_EnableHighOSC(E_INTERNAL,10);       //Select HSRC
  DrvCLOCK_SelectIHOSC(1);                     //Select internal 4MHZ
  DrvCLOCK_SelectMCUClock(0,0);                //CPU CLOCK IS 'hs_ck/1'
  DrvCLOCK_CalibrateHAO(1);                    //Calibration 4.147MHz
  #endif
  #if defined(HAO_10MHZ)
  //Clock INIT 10MHZ
  DrvCLOCK_EnableHighOSC(E_INTERNAL,10);       //Select HSRC
  DrvCLOCK_SelectIHOSC(2);                     //Select internal 10MHZ
  DrvCLOCK_SelectMCUClock(0,0);                //CPU CLOCK IS 'hs_ck/1'
  DrvCLOCK_CalibrateHAO(2);                    //Calibration 9.216MHz
  #endif
  #if defined(HAO_16MHZ)
  //Clock INIT 16MHZ
  DrvCLOCK_EnableHighOSC(E_INTERNAL,10);       //Select HSRC
  DrvCLOCK_SelectIHOSC(3);                     //Select internal 16MHZ
  DrvCLOCK_SelectMCUClock(0,0);                //CPU CLOCK IS 'hs_ck/1'
  DrvCLOCK_CalibrateHAO(3);                    //Calibration 15.667MHz
  #endif
#endif

#if defined(HSXT)
  DrvCLOCK_EnableHighOSC (E_EXTERNAL,50);
#endif
}

/*----------------------------------------------------------------------------*/
/* Function Name: InitalUart2()                                               */
/* Description  : UART2 Initial Subroutines.                                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalUart2(void)
{

#if defined(HSRC)
  DrvUART2_ClkEnable(1,0);            //Choose the internal HAO as clock source
  #if defined(PORT9)
    #if defined(HAO_2MHZ)
    DrvUART2_Open(1843,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,7);
    #endif
    #if defined(HAO_4MHZ)
    DrvUART2_Open(4147,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,7);
    #endif
    #if defined(HAO_10MHZ)
    DrvUART2_Open(9216,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,7);
    #endif
    #if defined(HAO_16MHZ)
    DrvUART2_Open(15667,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,7);
  #endif
  #else //PORT2
    #if defined(HAO_2MHZ)
    DrvUART2_Open(1843,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
    #endif
    #if defined(HAO_4MHZ)
    DrvUART2_Open(4147,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
    #endif
    #if defined(HAO_10MHZ)
    DrvUART2_Open(9216,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
    #endif
    #if defined(HAO_16MHZ)
    DrvUART2_Open(15667,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
    #endif
  #endif
#endif
  //1843 : oscillator frequency 4MHz Unit After Calibration HAO = 1843kHz
  //4147 : oscillator frequency 4MHz Unit After Calibration HAO = 4147kHz
  //9216 : oscillator frequency 4MHz Unit After Calibration HAO = 9216kHz
  //15667 : oscillator frequency 4MHz Unit After Calibration HAO = 15667kHz
  //None parity
  //8 data bits.
  //7 : Port 9.6 =TX, Port 9.7 =RX
  //2 : Port 2.2 =TX, Port 2.3 =RX

#if defined(HSXT)
  DrvUART2_ClkEnable(0,0);            //Choose the external OSC as clock source
  DrvUART2_Open(4000,B9600,DRVUART_PARITY_NONE,DRVUART_DATABITS_8,DRVUART_STOPBITS_1,2);
#endif

#if defined(PORT9)
  DrvGPIO_LCDIOOpen(UART2_PORT, UART2_TXD, E_IO_OUTPUT);
  DrvGPIO_LCDIOOpen(UART2_PORT, UART2_RXD, E_IO_INPUT);

#else
  DrvGPIO_Open(UART2_PORT,UART2_TXD,E_IO_OUTPUT);
  DrvGPIO_Open(UART2_PORT,UART2_RXD,E_IO_INPUT);
  DrvGPIO_Open(UART2_PORT,UART2_RXD|UART2_TXD,E_IO_PullHigh);
#endif

  DrvUART2_EnableInt(ENABLE,ENABLE); //ENABLE UART2 Tx Interrupt, ENABLE UART2 Rx Interrupt
  DrvUART2_Disable_AutoBaudrate();
  DrvUART2_Close();
  DrvUART2_Enable();
}

/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}


/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
